import 'package:clean_android_design/clean_android_design.dart';
import 'package:clean_android_design/src/settings/settings.dart';
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

class SelectableListFieldWidget<TMain, TItem extends ReadOnlyDatabaseItem>
    extends StatefulWidget {
  final bool isFirst;
  final bool isLast;

  final void Function(List<DocumentReference> selectedItemRefs) saveValues;
  final List<DocumentReference> Function() getValues;

  final bool Function(TItem) allItemsLambda;
  final ISelectableListItem Function(BuildContext context, TItem item)
      itemBuilder;
  final Comparable Function(TItem item) sortProperty;

  final bool bound;
  final bool isEditable;

  final Widget noItemWidget;
  final Widget noItemWidgetEditable;

  final Widget topWidget;
  final Widget title;

  final ForceUpdateKey activateKeyOnChange;

  const SelectableListFieldWidget({
    Key key,
    this.saveValues,
    this.getValues,
    this.allItemsLambda,
    this.itemBuilder,
    this.noItemWidget,
    this.noItemWidgetEditable,
    this.sortProperty,
    this.topWidget,
    this.title,
    this.activateKeyOnChange,
  })  : isFirst = false,
        isLast = false,
        bound = true,
        isEditable = null,
        assert(getValues != null),
        assert(saveValues != null),
        super(key: key);

  const SelectableListFieldWidget.unbound({
    Key key,
    this.saveValues,
    this.getValues,
    this.allItemsLambda,
    this.isEditable,
    this.itemBuilder,
    this.noItemWidget,
    this.noItemWidgetEditable,
    this.sortProperty,
    this.topWidget,
    this.title,
    this.activateKeyOnChange,
  })  : isFirst = false,
        isLast = false,
        bound = false,
        assert(getValues != null),
        assert(saveValues != null),
        super(key: key);

  @override
  _SelectableListFieldWidgetState<TMain, TItem> createState() =>
      _SelectableListFieldWidgetState<TMain, TItem>();
}

class _SelectableListFieldWidgetState<TMain, TItem extends ReadOnlyDatabaseItem>
    extends State<SelectableListFieldWidget<TMain, TItem>> {
  List<TItem> allItems;

  @override
  Widget build(BuildContext context) {
    allItems = Provider.of<List<TItem>>(context)
        .where((item) => widget.allItemsLambda?.call(item) ?? true)
        .toList();

    if (widget.sortProperty != null)
      allItems.sort((i1, i2) =>
          widget.sortProperty(i1).compareTo(widget.sortProperty(i2)));

    if (widget.isEditable == null)
      return Consumer<bool>(
          builder: (context, isEditable, __) =>
              _getDefautFieldWidget(isEditable));
    else
      return _getDefautFieldWidget(widget.isEditable);
  }

  Widget _getDefautFieldWidget(bool isEditable) {
    var items = isEditable
        ? allItems
        : allItems
            .where((e) => widget.getValues().any((ref) => ref.id == e.id))
            .toList();

    if (items.length == 0) if (isEditable)
      return widget.noItemWidgetEditable ?? widget.noItemWidget ?? Container();
    else
      return widget.noItemWidget ?? Container();

    return Column(
      children: <Widget>[
        widget.topWidget ?? Container(),
        ItemSectionWidget.builder(
          title: widget.title,
          itemBuilder: (context, index) => widget
              .itemBuilder(context, items[index])
              .copyWithSelection(
                isSelected: isEditable
                    ? widget.getValues().any((ref) => ref.id == items[index].id)
                    : false,
                onTapList: isEditable
                    ? () {
                        _changeSelectionFor(items[index]);
                        return false;
                      }
                    : null,
              ),
          itemCount: items.length,
        ),
      ],
    );
  }

  _changeSelectionFor(TItem item) {
    var selectedItems = widget.getValues();
    if (selectedItems.any((ref) => ref.id == item.id))
      selectedItems.removeWhere((ref) => ref.id == item.id);
    else
      selectedItems.add(CleanAndroidDesignSettings.firestore
          .doc("${TItem.toString()}/${item.id}"));

    setState(() => widget.saveValues(selectedItems));
    widget.activateKeyOnChange?.update();
  }
}
