import 'dart:core';

import 'package:flutter/material.dart';

class Validator<T> implements IValidation<T> {
  final Key pageKey;

  const Validator() : pageKey = null;

  const Validator.forPage({@required this.pageKey});

  final List<IValidation<T>> _children = const <IValidation<T>>[];

  void addChild(IValidation validation) {
    _children.add(validation);
  }

  ValidationResult isValid(T item) {
    var result = ValidationResult(pageKey: pageKey);

    _children.forEach((validation) => result.merge(validation?.isValid(item)));
    return result;
  }
}

abstract class IValidation<T> {
  ValidationResult isValid(T item);
}

class ValidationResult {
  bool isValid;
  FocusNode firstInvalidNode;
  Key firstInvalidPage;

  ValidationResult({this.isValid = true, FocusNode focusNode, Key pageKey})
      : this.firstInvalidNode = focusNode,
        this.firstInvalidPage = pageKey;

  ValidationResult merge(ValidationResult result) {
    this.isValid = this.isValid && result.isValid;
    if (!result.isValid) {
      this.firstInvalidNode ??= result.firstInvalidNode;
      this.firstInvalidPage ??= result.firstInvalidPage;
    }
    return this;
  }
}
